@@ -3,6 +3,7 @@
 import threading
 import time
 import struct
+import math
 try:
   import Queue
 except:
@@ -62,7 +63,10 @@ class Linkbot(mobot.Mobot):
 
   def connectBluetooth(self, bluetooth_mac_addr):
     """
-    Connect to a bluetooth enabled Linkbot.
+    Connect to a bluetooth enabled Linkbot. Note that you must have a bluetooth
+    enabled LinkPod connected to your Linkbot for this function to work. Most
+    Linkbots do not come with BlueTooth, and instead use a ZigBee like protocol
+    to communicate with each other.
 
     @type bluetooth_mac_addr: string
     @param bluetooth_mac_addr: The MAC address of the bluetooth Linkbot. Should
@@ -85,6 +89,39 @@ class Linkbot(mobot.Mobot):
   def driveToNB(self, angle1, angle2, angle3):
     mobot.Mobot.driveToNB(self, angle1, angle2, angle3, 0)
 
+  def disableAccelEventCallback(self):
+    buf = bytearray([barobo.BaroboCtx.CMD_SET_ENABLE_ACCEL_EVENT, 4, 0x00, 0x00])
+    self._transactMessage(buf)
+    self.accelCallbackEnabled = False
+
+  def disableJointEventCallback(self):
+    buf = bytearray([barobo.BaroboCtx.CMD_SET_ENABLE_JOINT_EVENT, 4, 0x00, 0x00])
+    self._transactMessage(buf)
+    self.jointCallbackEnabled = False
+
+  def enableJointEventCallback(self, cb):
+    """ 
+    Enable the joint event callback. Whenever any joint on the robot moves, the 
+    callback function 'cb' will be called. The callback function should be
+    of the form cb(millis_timestamp, j1, j2, j3) where 'millis_timestamp' 
+    will contain a timestamp from the robot, and j1, j2, and j3 will contain
+    the new angles of all of the joints in degrees. 
+    """
+    buf = bytearray([barobo.BaroboCtx.CMD_SET_ENABLE_JOINT_EVENT, 4, 0x07, 0x00])
+    self._transactMessage(buf)
+    self.jointcallbackfunc = cb
+    self.jointCallbackEnabled = True
+
+  def enableAccelEventCallback(self, cb):
+    """
+    Enable the acceleration event callback. Whenever a change in acceleration
+    happens on the robot, the callback function will be called.
+    """
+    buf = bytearray([barobo.BaroboCtx.CMD_SET_ENABLE_ACCEL_EVENT, 4, 0x07, 0x00])
+    self._transactMessage(buf)
+    self.accelcallbackfunc = cb
+    self.accelCallbackEnabled = True
+
   def _getADCVolts(self, adc):
     buf = bytearray([barobo.BaroboCtx.CMD_GETENCODERVOLTAGE, 4, adc, 0])
     response = self._transactMessage(buf)
@@ -237,7 +274,7 @@ class Linkbot(mobot.Mobot):
     buf += bytearray([0x00])
     buf[1] = len(buf)
     response = self._transactMessage(buf, maxTries = 1, timeout = 0.5)
-    if response[2:-1] != randbytes:
+    if response[2:2+numbytes] != randbytes:
       raise barobo.BaroboException(
           'Ping did not receive correct bytes. Expected {0}, got {1}'.format(
             list(map(hex, randbytes)),
@@ -323,7 +360,7 @@ class Linkbot(mobot.Mobot):
     """
     Set the reference voltage of an analog input pin. 
 
-    @type pin: integer from 0-7
+    @type ref: integer from 0-7
     @param ref: Valid values are barobo.AREF_DEFAULT, barobo.AREF_INTERNAL,
       barobo.AREF_INTERNAL1V1, barobo.AREF_INTERNAL2V56, and
       barobo.AREF_EXTERNAL.
@@ -359,6 +396,40 @@ class Linkbot(mobot.Mobot):
     buf = bytearray([0x6A, 0x05, (freq>>8)&0xff, freq&0xff, 0x00])
     self._transactMessage(buf)
 
+  def setAccelEventThreshold(self, g_force):
+    """
+    Set the minimum change in g-force required for the robot to report an
+    acceleration event.
+
+    See also: enableAccelEventCallback()
+
+    @type g_force: floating point number
+    @param g_force: The acceleration in standard earth gravity "G's".
+    """
+    assert g_force >= 0
+    buf = bytearray([barobo.BaroboCtx.CMD_SET_ACCEL_EVENT_THRESHOLD,
+        0])
+    buf += bytearray(struct.pack('!H', g_force * 16384))
+    buf += bytearray([0x0])
+    self._transactMessage(buf)
+
+  def setJointEventThreshold(self, angle):
+    """
+    Set the minimum amount the joint must move before a joint motion event is
+    reported.
+
+    Also see enableJointEventCallback()
+
+    @type angle: floating point number
+    @param angle: An angle in degrees.
+    """
+    assert angle >= 0
+    buf = bytearray([barobo.BaroboCtx.CMD_SET_JOINT_EVENT_THRESHOLD,
+        0])
+    buf += bytearray(struct.pack('<f', angle*math.pi/180.0))
+    buf += bytearray([0x0])
+    self._transactMessage(buf)
+
   def setHWRev(self, major, minor, micro):
     """
     Set the HW revision of the Linkbot.
